library nb_utils;

import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:nb_utils/src/colors.dart';

export 'package:connectivity/connectivity.dart';
export 'package:fluttertoast/fluttertoast.dart';
export 'package:shared_preferences/shared_preferences.dart';

export 'src/Marquee.dart';
export 'src/colors.dart';
export 'src/constants.dart';
export 'src/context_extensions.dart';
export 'src/date_time_utils.dart';
export 'src/device_extensions.dart';
export 'src/livestream.dart';
export 'src/network_utils.dart';
export 'src/number_extensions.dart';
export 'src/scroll_extensions.dart';
export 'src/shared_pref.dart';
export 'src/string_extensions.dart';
export 'src/text_extensions.dart';
export 'src/time_formatter.dart';
export 'src/widget_extensions.dart';
export 'src/widgets.dart';
export 'src/decorations.dart';
export 'src/extensions.dart';

// Global variables - This variables can be changed.
Color textPrimaryColorGlobal = textPrimaryColor;
Color textSecondaryColorGlobal = textSecondaryColor;
Color shadowColorGlobal = Colors.black12;
Color appBarBackgroundColorGlobal = Colors.white;
double defaultRadius = 8.0;
double defaultBlurRadius = 5.0;
//

/// Go back to previous screen.
finish(context, [Object result]) => Navigator.pop(context, result);

/// Go to new screen with provided screen tag.
///
/// ```dart
/// launchNewScreen(context, '/HomePage');
/// ```
launchNewScreen(context, tag) => Navigator.of(context).pushNamed(tag);

/// Removes all previous screens from the back stack and redirect to new screen with provided screen tag
///
/// ```dart
/// launchNewScreenWithNewTask(context, '/HomePage');
/// ```
launchNewScreenWithNewTask(context, tag) => Navigator.of(context).pushNamedAndRemoveUntil(tag, (r) => false);

/// Returns Color from hex String.
///
/// ```dart
/// Color color = getColorFromHex('#E5E5E5');
/// ```
Color getColorFromHex(String hexColor, {Color defaultColor}) {
  try {
    return Color(int.parse(hexColor.substring(1, 7), radix: 16) + 0xFF000000);
  } catch (e) {
    print(e);
    return defaultColor ?? Colors.white;
  }
}

/// Hide soft keyboard
hideKeyboard(context) => FocusScope.of(context).requestFocus(FocusNode());

mixin AfterLayoutMixin<T extends StatefulWidget> on State<T> {
  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addPostFrameCallback((_) => afterFirstLayout(context));
  }

  void afterFirstLayout(BuildContext context);

  @override
  void dispose() {
    super.dispose();
  }
}

// Styles

TextStyle boldTextStyle({
  int size = 16,
  Color color,
  FontWeight weight = FontWeight.bold,
  String fontFamily,
  double letterSpacing,
  FontStyle fontStyle,
  double wordSpacing,
  TextDecoration decoration,
  TextDecorationStyle textDecorationStyle,
  TextBaseline textBaseline,
}) {
  return TextStyle(
    fontSize: size.toDouble(),
    color: color ?? textPrimaryColorGlobal,
    fontWeight: weight,
    fontFamily: fontFamily,
    letterSpacing: letterSpacing,
    fontStyle: fontStyle,
    decoration: decoration,
    decorationStyle: textDecorationStyle,
    wordSpacing: wordSpacing,
    textBaseline: textBaseline,
  );
}

//
TextStyle primaryTextStyle({
  int size = 16,
  Color color,
  FontWeight weight = FontWeight.normal,
  String fontFamily,
  double letterSpacing,
  FontStyle fontStyle,
  double wordSpacing,
  TextDecoration decoration,
  TextDecorationStyle textDecorationStyle,
  TextBaseline textBaseline,
}) {
  return TextStyle(
    fontSize: size.toDouble(),
    color: color ?? textPrimaryColorGlobal,
    fontWeight: weight,
    fontFamily: fontFamily,
    letterSpacing: letterSpacing,
    fontStyle: fontStyle,
    decoration: decoration,
    decorationStyle: textDecorationStyle,
    wordSpacing: wordSpacing,
    textBaseline: textBaseline,
  );
}

//
TextStyle secondaryTextStyle({
  int size = 14,
  Color color,
  FontWeight weight = FontWeight.normal,
  String fontFamily,
  double letterSpacing,
  FontStyle fontStyle,
  double wordSpacing,
  TextDecoration decoration,
  TextDecorationStyle textDecorationStyle,
  TextBaseline textBaseline,
}) {
  return TextStyle(
    fontSize: size.toDouble(),
    color: color ?? textSecondaryColorGlobal,
    fontWeight: weight,
    fontFamily: fontFamily,
    letterSpacing: letterSpacing,
    fontStyle: fontStyle,
    decoration: decoration,
    decorationStyle: textDecorationStyle,
    wordSpacing: wordSpacing,
    textBaseline: textBaseline,
  );
}

//

RichText createRichText(List<TextSpan> list, {TextOverflow overflow = TextOverflow.ellipsis, int maxLines, TextAlign textAlign = TextAlign.left}) {
  return RichText(text: TextSpan(children: list), overflow: overflow, maxLines: maxLines, textAlign: textAlign);
}

toast(String value, {ToastGravity gravity = ToastGravity.BOTTOM, length = Toast.LENGTH_SHORT, Color bgColor, Color textColor}) {
  Fluttertoast.showToast(msg: value, gravity: gravity, toastLength: length, backgroundColor: bgColor, textColor: textColor);
}

toastLong(String value, {ToastGravity gravity = ToastGravity.BOTTOM, length = Toast.LENGTH_LONG, Color bgColor, Color textColor}) {
  toast(value, gravity: gravity, bgColor: bgColor, textColor: textColor, length: length);
}

log(Object value) {
  var dateTime = DateTime.now();
  //print('${dateTime.hour}:${dateTime.minute}:${dateTime.second} $value');

  debugPrint('${dateTime.hour}:${dateTime.minute}:${dateTime.second} $value');
}

/// Returns MaterialColor from Color
MaterialColor createMaterialColor(Color color) {
  List strengths = <double>[.05];
  Map swatch = <int, Color>{};
  final int r = color.red, g = color.green, b = color.blue;

  for (int i = 1; i < 10; i++) {
    strengths.add(0.1 * i);
  }
  strengths.forEach((strength) {
    final double ds = 0.5 - strength;
    swatch[(strength * 1000).round()] = Color.fromRGBO(
      r + ((ds < 0 ? r : (255 - r)) * ds).round(),
      g + ((ds < 0 ? g : (255 - g)) * ds).round(),
      b + ((ds < 0 ? b : (255 - b)) * ds).round(),
      1,
    );
  });
  return MaterialColor(color.value, swatch);
}
