import 'package:flutter/material.dart';

import '../nb_utils.dart';

Future showConfirmDialog<bool>(context, String title, {positiveText = 'Yes', negativeText = 'No', Color buttonColor}) async {
  return showDialog(
    context: context,
    barrierDismissible: false,
    child: AlertDialog(
      title: Text(title),
      actions: <Widget>[
        SimpleDialogOption(
          child: Text(negativeText, style: secondaryTextStyle()),
          onPressed: () {
            Navigator.of(context).pop(false);
          },
        ),
        SimpleDialogOption(
          onPressed: () {
            Navigator.of(context).pop(true);
          },
          child: Text(positiveText, style: primaryTextStyle(color: buttonColor ?? Theme.of(context).primaryColor)),
        ),
      ],
    ),
  );
}

Future<dynamic> showInDialog(
  BuildContext context, {
  Widget title,
  Widget child,
  ShapeBorder shape,
  TextStyle titleTextStyle,
  EdgeInsetsGeometry contentPadding = const EdgeInsets.fromLTRB(24.0, 20.0, 24.0, 24.0),
  //bool scrollable = false,
  Color backgroundColor,
  double elevation,
  //EdgeInsets insetPadding = const EdgeInsets.symmetric(horizontal: 16.0, vertical: 16.0),
  List<Widget> actions,
  bool barrierDismissible = true,
}) async {
  return await showDialog(
    context: context,
    barrierDismissible: barrierDismissible,
    child: AlertDialog(
      content: child,
      shape: shape,
      title: title,
      titleTextStyle: titleTextStyle,
      contentPadding: contentPadding,
      //scrollable: scrollable,
      backgroundColor: backgroundColor,
      elevation: elevation,
      //insetPadding: insetPadding,
      actions: actions,
    ),
  );
}

AppBar appBarWidget(
  String title, {
  List<Widget> actions,
  Color color,
  bool center = false,
  automaticallyImplyLeading = true,
  Color textColor,
  int textSize = 20,
  bool showBack = true,
  Color shadowColor,
}) {
  return AppBar(
    centerTitle: center,
    title: Text(title, style: boldTextStyle(color: textColor ?? textPrimaryColorGlobal, size: textSize)),
    actions: actions,
    automaticallyImplyLeading: automaticallyImplyLeading,
    backgroundColor: color ?? appBarBackgroundColorGlobal,
    leading: showBack ? BackButton() : null,
    shadowColor: shadowColor,
  );
}

enum SymbolType { Bullet, Numbered }

class UL extends StatelessWidget {
  List<Widget> children;
  double padding;
  double spacing;
  SymbolType symbolType;
  Color symbolColor;
  Color textColor;

  UL({
    this.children = const <Widget>[],
    this.padding = 8,
    this.spacing = 8,
    this.symbolType = SymbolType.Bullet,
    this.symbolColor,
    this.textColor,
  });

  @override
  Widget build(BuildContext context) {
    return Wrap(
      runSpacing: spacing,
      direction: Axis.horizontal,
      children: children
          .asMap()
          .map((i, element) => MapEntry(
                i,
                Row(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    symbolType == SymbolType.Bullet ? Text('•', style: TextStyle(fontSize: 30, color: textColor ?? textPrimaryColorGlobal)) : SizedBox(),
                    symbolType == SymbolType.Numbered ? Text('${i + 1}', style: TextStyle(fontSize: 16, color: symbolColor ?? textPrimaryColorGlobal)) : SizedBox(),
                    SizedBox(width: padding),
                    element.expand(),
                  ],
                ),
              ))
          .values
          .toList(),
    );
  }
}
